local _G = _G;
local strformat, pairs, ipairs, type = _G.string.format, _G.pairs, _G.ipairs, _G.type;

local COLORS = {
  ARTIFACT_GOLD   = _G.ARTIFACT_BAR_COLOR;
  BATTLE_NET_BLUE = _G.BATTLENET_FONT_COLOR;
  BLACK           = _G.BLACK_FONT_COLOR;
  DIM_GREEN       = _G.DIM_GREEN_FONT_COLOR;
  DIM_RED         = _G.DIM_RED_FONT_COLOR;
  DULL_RED        = _G.DIM_RED_FONT_COLOR;
  GOLD            = _G.NORMAL_FONT_COLOR;
  GRAY            = _G.DISABLED_FONT_COLOR;
  GREEN           = _G.GREEN_FONT_COLOR;
  LIGHT_YELLOW    = _G.LIGHTYELLOW_FONT_COLOR;
  ORANGE          = _G.ORANGE_FONT_COLOR;
  PARTY_CHAT_BLUE = _G.LIGHTBLUE_FONT_COLOR;
  RED             = _G.RED_FONT_COLOR;
  TRANSMOG_VIOLET = _G.TRANSMOGRIFY_FONT_COLOR;
  WHITE           = _G.HIGHLIGHT_FONT_COLOR;
  YELLOW          = _G.YELLOW_FONT_COLOR;
};

local wordsOfInterest = {
  {
    "healers", "healer", "healz", "heal",
    color = "GREEN"
  },
  {
    "tanks", "tank",
    color = "RED"
  },
  ["dps"] = "YELLOW";
};

local function GetStartAndEnd(query, word)
  if (type(word) == "table") then
    local startId, endId;
    local smallestStartId, smallestEndId;

    for _, w in ipairs(word) do
      startId, endId = query:find(w:lower());

      if (startId and endId) then
        if (not smallestStartId or startId < smallestStartId) then
          smallestStartId = startId;
          smallestEndId = endId;
        end
      end
    end

    return smallestStartId, smallestEndId;
  end

  return query:find(word:lower());
end

local function HighlightSubStringsByKey(msg, word, colorKey, upperCase)
  local query = msg:lower();
  local offset = 0;

  while (true) do
    local startId, endId = GetStartAndEnd(query, word);
    if (not (startId and endId)) then break end

    local pre = msg:sub(0, startId - 1 + offset) -- everything before
    local found = msg:sub(startId + offset, endId + offset);
    found = COLORS[colorKey:upper()]:WrapTextInColorCode(upperCase and found:upper() or found);
    local post = msg:sub(endId + 1 + offset, #msg); -- everything after

    query = post:lower();
    offset = (#(pre .. found));

    msg = pre .. found .. post;
  end

  return msg;
end

local function GetChatLink(url)
	return strformat("|Hurl:%s|h|cffffe29e%s|r|h", url, "["..url.."]");
end

local function NewAddMessage(self, text, ...)
  if (not text) then return; end

  local prefix, body = text:match("(|Hchannel:.-|h )(.*)");

  if (prefix and body) then
    body = body:trim();

    -- highlight words of interest:
    for word, value in pairs(wordsOfInterest) do
      if (type(value) == "table") then
        body = HighlightSubStringsByKey(body, value, value.color, true);
      else
        body = HighlightSubStringsByKey(body, word, value, true);
      end
    end

    text = prefix .. body;
  end

  self:oldAddMessage(text:gsub("[wWhH][wWtT][wWtT][\46pP]%S+[^%p%s]", GetChatLink), ...);
end

for _, chatFrameName in ipairs(_G.CHAT_FRAMES) do
  local chatFrame = _G[chatFrameName];

  if (chatFrame:GetID() ~= 2) then
    -- if not combat log...
    chatFrame.oldAddMessage = chatFrame.AddMessage;
    chatFrame.AddMessage = NewAddMessage;
  end
end